/* Copyright (c) 1999 TTK Ciar / Bill Moyer
 * All rights reserved.
 *
 * This code is derived from code developed by TTK Ciar and Bill Moyer.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer.
 *  2. The name of the author may not be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

int major_revision = 3;
int minor_revision = 0;

char *fname;      /* name of file to review */
char *pname;      /* name of this executable */
int   width;      /* number of bytes processed per line */
int   hexcase;    /* 0 for lowercase hex, 1 for uppercase */
int   useheader;  /* 1 to generate filename/filesize prefix line */
int   use_stdin;  /* 0 for filesystem file, 1 for stdin */

void usage() {
  printf ("usage: %s [-h] [-C] [-n <name>] [-v] [-w <##>] [<filename>]\n",pname);
  printf ("\t-h  turns off emission of name/size header line\n");
  printf ("\t-C  use uppercase hexidecimal\n");
  printf ("\t-n  use <name> in name/size header line\n");
  printf ("\t-v  print version information and exit\n");
  printf ("\t-w  display <##> bytes per line (default 8)\n");
  printf ("\tdumps contents of <filename> (or stdin) to stdout in hex\n");
  printf ("\tand ascii representation with addressing information.\n");
  }

void parse_args ( int ac, char **av ) {
  int i;
  pname = av[0];
  fname = NULL;
  width = 8;
  hexcase = 0;
  useheader = 1;
  use_stdin = 1;
  if ( ac == 1 ) return;
  for ( i = 1; i < ac; i++ ) {
    if ( av[i][0] == '-' ) switch ( av[i][1] ) {
      case  0  : break;
      case 'h' : useheader = 0; break;
      case 'C' : hexcase = 1; break;
      case 'n' : i++; if ( i >= ac ) return;
                 fname = av[i];
                 break;
      case 'v' : printf ("quickdump v%d.%d originally by TTK Ciar\n",major_revision,minor_revision); exit(0); break;
      case 'w' : i++; if ( i >= ac ) return; 
                 width = atol ( av[i] );
                 if ( width < 1 ) width = 1;
                 break;
      default : usage(); exit(0); break;
      }
    else { use_stdin = 0; fname = av[i]; }
    }
  }

int   pix = 0;
char  paddedstr[8][128];
char *padstr ( char *str, char pad, int w_ ) {
  int len, r, i, w;
  if ( pix >= 8 ) pix = 0;
  paddedstr[pix][0] = 0;
  if ( str == NULL ) return paddedstr[pix++];
  w = w_;
  if ( w > 127 ) return paddedstr[pix++];
  len = strlen(str);
  if ( len >= w ) { 
    strcpy ( paddedstr[pix], str ); 
    return ( paddedstr[pix++] );
    }
  r = w - len;
  for ( i = 0; i < r; i++ ) paddedstr[pix][i] = pad;
  strcpy ( &(paddedstr[pix][r]), str );
  return ( paddedstr[pix++] );
  }

char *padnum ( int num, int w ) {
  char numstr[32];
  sprintf ( numstr, "%d", num );
  return padstr ( numstr, '0', 2 );
  }

char *padhex ( int num, int w, int ucase ) {
  char numstr[32];
  if ( ucase ) 
       sprintf ( numstr, "%X", num );
  else sprintf ( numstr, "%x", num );
  return padstr ( numstr, '0', 2 );
  }

int main( ac, av ) int ac; char *av[];
  {  FILE *f;
     int  i, j, bit, quit, res;
     char *asciistr;
     int  idatasectionindex = 0;
     int  fsz;
     unsigned char foo;

     parse_args ( ac, av );
     asciistr = (char*) malloc ( width + 1 );
     asciistr[width] = 0;
     if ( use_stdin == 0 ) {
       f = fopen ( fname, "r" );
       if ( f == NULL ) { 
         fprintf (stderr,"%s: cannot open file '%s'\n",pname,fname); 
         exit(0); 
         }
       fseek ( f, 0, SEEK_END );
       fsz = ftell(f);
       fseek ( f, 0, SEEK_SET );
       }
     else { fname = "stdin"; f = stdin; fsz = 0; }
     if ( useheader ) printf ("[%s] %d\n",fname,fsz);
     i = 0; j = 0; quit = 0;
     while ( !quit ) {
       res = fread ( &foo, 1, 1, f );
       if ( res == 0 ) {
         foo = 0;
         if ( i % width == 0 ) quit = 1;
         }
       if ( i % width == 0 ) if ( !quit )
         printf ( "%6u %5X : ", j, j );
       asciistr[i] = foo;
       if ( ( (unsigned char)asciistr[i] < ' ' ) ||
            ( (unsigned char)asciistr[i] > 126 )    )
          asciistr[i] = '.';
       if ( !quit || i ) {
          if ( res == 0 ) printf (" **");
          else printf ("%3s",padhex(foo,2,hexcase));
          }
       i++; j++;
       if ( i % width == 0 ) { 
         asciistr[i] = 0;
         i = 0; 
         printf (" |%s\n",asciistr); 
         }
       }
     fclose ( f );
     return 0;
     }
